str_description = """

    The data models/schemas for the PACS QR collection.

"""

from    pydantic            import BaseModel, Field
from    typing              import Optional, List, Dict
from    datetime            import datetime
from    enum                import Enum
from    pathlib             import Path



class sensorSimple(BaseModel):
    """The simplest sensor model POST"""
    sensor                              : str   = ""

class sensorDelete(BaseModel):
    status                              : bool  = False

class sensorBoolReturn(BaseModel):
    status                              : bool  = False

class persairResponse(BaseModel):
    """The model returned internally"""
    status                              : bool  = False
    message                             : str   = ''
    response                            : dict  = {}

class time(BaseModel):
    """A simple model that has a time string field"""
    time                                : str




#!/usr/bin/env python3
#
# (c) 2023+ Fetal-Neonatal Neuroimaging & Developmental Science Center
#                   Boston Children's Hospital
#
#              http://childrenshospital.org/FNNDSC/
#                        dev@babyMRI.org
#

__version__ = "2.2.0"

import sys, os

from    persair             import persair

try:
    from    .               import __pkg, __version__
except:
    from persair           import __pkg, __version__

import  asyncio
from    asyncio             import AbstractEventLoop

from    argparse            import RawTextHelpFormatter
from    argparse            import ArgumentParser, Namespace
import  pudb
from    pfmisc              import Colors as C
from    print_color         import print as cprint
from    typing              import Any, Literal
import  json
import  re
NC =  C.NO_COLOUR

str_title:str = f"""

                                                   _
                                                  (_)
                         _ __   ___ _ __ ___  __ _ _ _ __
                        | '_ \ / _ \ '__/ __|/ _` | | '__|
                        | |_) |  __/ |  \__ \ (_| | | |
                        | .__/ \___|_|  |___/\__,_|_|_|
                        | |
                        |_|
"""

str_heading:str = f"""
                            python purpleair client

"""

def synopsis(ab_shortOnly = False) -> None:
    scriptName:str          = os.path.basename(sys.argv[0])
    print(C.CYAN + '''
    NAME
        ''', end = '' + NC)
    print(scriptName)
    print(C.CYAN + '''
    SYNPOSIS
        ''' + NC, end = '')
    print(scriptName + persair.package_CLIfull)
    print(C.CYAN + '''
    BRIEF EXAMPLE
    ''' + NC)

    if ab_shortOnly: return
    print(C.CYAN + '''
    ARGS''' + NC, end="")
    print(persair.package_argsSynopsisFull)

def earlyExit_check(args:Namespace) -> int:
    """
    Check if version/man page are required, and if so service
    and return

    Args:
        args (Namespace): The CLI namespace

    Returns:
        int: 0 -- continue
             1 -- exit
    """
    if args.man:
        cprint(str_title, color='green')
        str_help:str    = ""
        if args.man:    synopsis(False)
        else:           synopsis(True)
        return 1
    if args.b_version:
        print("Name:    ", end="")
        print(C.LIGHT_CYAN + f'{__pkg.name}' + NC)
        print("Version: ", end="")
        print(C.LIGHT_GREEN + f'{__version__}\n')
        return 1
    return 0

parser:ArgumentParser   = persair.parser_setup(persair.description)
pargs: Namespace        = persair.parser_interpret(parser)

def main(*args) -> Literal[1, 0]:

    # pudb.set_trace()
    # Preliminary setup
    if not args:
        argsToPass:Namespace                = pargs
    else:
        argsToPass:Namespace                = args[0]
    if earlyExit_check(argsToPass): return 1

    # Do something
    d_persair:dict          = {}
    sensors:persair.Persair = persair.Persair(argsToPass)

    loop:AbstractEventLoop  = asyncio.get_event_loop()
    loop.run_until_complete(sensors.service())

    # d_response:dict              = asyncio.run(sensors.service())
    print(sensors.responseData.json())

    return 0

if __name__ == "__main__":
    sys.exit(main())



import  os
from    pydantic_settings  import BaseSettings

class Keys(BaseSettings):
    DBauthPath:str      = '/db/init.json'
    ReadWriteKey:str    = "local"

class Mongo(BaseSettings):
    MD_URI:str          = "localhost:27017"
    MD_DB:str           = "default"
    MD_username:str     = "username"
    MD_password:str     = "password"


keys            = Keys()
mongosettings   = Mongo()



from os import write
from    typing              import Any, List, TypedDict
from    pydantic            import BaseModel, Field

import  json
from    datetime            import datetime
from    pathlib             import Path

try:
    from    config              import settings
except:
    from    ..config             import settings
import  sys
import  shutil
import  pudb

import  pymongo
from    pymongo             import MongoClient
from    pymongo.database    import Database
from    pymongo.collection  import Collection

class PFdb_mongo():
    """
    A mongo DB wrapper/interface object
    """

    def APIkeys_readFromFile(self, keyName:str) \
        -> dict[str, bool | str | dict[str, dict[Any, Any]]]:
        """
        Read a read/write key pair from a named <keyName> in
        the db init file (if it exists).

        Args:
            keyName (str): the key name to read in the db init.

        Return
            dict[str, bool | dict[Any, Any]]: The Read/Write key pair with
                                              bool 'status'
        """
        d_keys:dict     =   {
            'status'    : False,
            'message'   : 'DB init file not found.',
            'init':     {
                'keys'  : {}
            },
            'keyName'   : keyName
        }
        d_data:dict     = {}
        if not self.keyInitPath.is_file():
            return d_keys
        with open(str(self.keyInitPath), 'r') as f:
            try:
                d_data:dict = json.load(f)
                if keyName in d_data:
                    d_data[keyName]['readwritekeys']  = keyName
                    d_keys['status']        = True
                    d_keys['init']['keys']  = d_data[keyName]
                    d_keys['message']       = f'<keyName> "{keyName}" successfully loaded.'
                else:
                    d_keys['message']   = f'Init data does not have <keyName> {keyName}.'
            except:
                d_keys['message']   = f'Could not interpret key file {self.keyInitPath}.'
        return d_keys

    def Mongo_connectDB(self, DBname:str) -> dict:
        """
        Connect / create the DB.

        Args:
            DBname (str): the DB name

        Returns:
            dict[str, bool | Database[Any]]: DB -- the database
                                             bool -- False if DB is not yet created
        """
        d_ret:dict  = {
            'status':   True if DBname in self.Mongo.list_database_names() else False,
            'DB':       self.Mongo[DBname]
        }
        return d_ret

    def Mongo_connectCollection(self, mongocollection:str) -> dict:
        """
        Simply connect to a named "collection" in a mongoDB and return
        the collection and its status.

        :param mongocollection: the name of the collection
        :return: a dictionary with the collection and a status
        """
        d_ret:dict  = {
            'status':       True if mongocollection in self.DB.list_collection_names() else False,
            'collection':   self.DB[mongocollection]
        }
        return d_ret

    def readwriteKeys_inCollectionGet(
            self,
            d_readwrite:dict,
            collectionExists:bool
    ) -> dict|None:
        if not collectionExists:
            self.collection.insert_one(d_readwrite['init']['keys'])
        d_collectionData    = self.collection.find_one({'readwritekeys': d_readwrite['keyName']})
        return d_collectionData

    def key_get(self, name:str) -> dict:
        """
        Get an access "key" from the main class. This explictly returns a
        dictionary since the self.key member variable can be either dict or
        None which can be flagged by the LSP.

        :param name: the key "name" to lookup
        :return: a dictionary containing the key value (or an error dictionary)
        """
        ret:dict    = {
                "error": f"key {name} not found"
        }
        if self.keys:
            if name in self.keys:
                ret = self.keys[name]
        return ret

    def __init__(self,
                 settingsKeys: settings.Keys,
                 settingsMongo: settings.Mongo) -> None:
        """
        Main database constructor.

        :param settingsKeys: a collection of default configuration settings
        :param settingsMongo: a collection of settings relevant to the mongoBD
        :return: the object
        """

        self.keyInitPath        = Path(settingsKeys.DBauthPath)
        self.Mongo              = MongoClient(settingsMongo.MD_URI,
                                              username  = settingsMongo.MD_username,
                                              password  = settingsMongo.MD_password)

        # Read the API read/write keys from self.keyInitPath
        # and ReadWriteKey collection
        # --- this is used only to instantiate the keys in the monogoDB
        d_readwrite: dict[str, bool | str | dict[Any, Any]] = \
            self.APIkeys_readFromFile(settingsKeys.ReadWriteKey)

        # Connect to the DB
        self.DB:Database[Any]               = self.Mongo_connectDB(settingsMongo.MD_DB)['DB']

        # Connect to the collection
        d_collection:dict                   = self.Mongo_connectCollection('sensors')
        self.collection:Collection[Any]     = d_collection['collection']
        self.keys = self.readwriteKeys_inCollectionGet(d_readwrite, d_collection['status'])

